"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AddReccord = void 0;
const errors_1 = require("../utils/errors");
const db_1 = require("../utils/db");
const uuid_1 = require("uuid");
class AddReccord {
    constructor(obj) {
        if (!obj.name || obj.name.length > 100) {
            throw new errors_1.ValidationError('Nazwa nie może być pusta ani przekraczać 100 znaków');
        }
        if (obj.description.length > 1024) {
            throw new errors_1.ValidationError('Opis nie może być dłuższy niz 1000znaki');
        }
        if (obj.price < 0 || obj.price > 999999) {
            throw new errors_1.ValidationError('Cena nie może być większa niż 999999');
        }
        if (obj.url.length < 1) {
            throw new errors_1.ValidationError('Musisz podać link do ogłoszenia');
        }
        if (typeof obj.lat !== 'number' || typeof obj.lon !== "number") {
            throw new errors_1.ValidationError('Nie można zlokalizować ogłoszenia');
        }
        this.name = obj.name;
        this.description = obj.description;
        this.id = obj.id;
        this.lon = obj.lon;
        this.lat = obj.lat;
        this.price = obj.price;
        this.url = obj.url;
    }
    static getOne(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `ads` WHERE id= :id", {
                id
            });
            return results.length === 0 ? null : new AddReccord(results[0]);
        });
    }
    static findAll(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `ads` WHERE `name` LIKE :search", {
                search: `%${name}%`,
            });
            return results.map(result => {
                const { id, lat, lon } = result;
                return { id, lat, lon };
            });
        });
    }
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.id) {
                this.id = (0, uuid_1.v4)();
            }
            else {
                throw new Error("cannot insert same id!");
            }
            ;
            yield db_1.pool.execute("INSERT INTO `ads`(`id`, `name`, `description`, `price`, `url`, `lat`, `lon`) VALUES(:id, :name, :description, :price, :url, :lat, :lon)", this);
        });
    }
}
exports.AddReccord = AddReccord;
//# sourceMappingURL=add.reccord.js.map